function cisldm=cisLDM(x, y, cost, params, op, inx)

% training a classifier using cisldm
% (x, y): training set with n examples, x:n-by-d, y:n-by-1
% cost: cost interval
% params: parameters used for paramter selection 
%              -- 'ker': kernel parameters
%              -- 'C1': C1 parameter in Eq. 14
%              -- 'C2': C2 parameter in Eq. 14
%              -- 'lambda1': lambda1 parameter in Eq. 14
%              -- 'lambda2': lambda2 parameter in Eq. 14
% op: option for solver type
%    0 -- Coordinate Descent
%    1 -- Average Stochastic Gradient Descent
% inx: index of labeled examples
% cisldm: an cisldm, structure type with the following fields:
%          -- 'x':x
%          -- 'ker':ker
%          -- 'alpha': alpha values
% 
% Developed with Matlab R2014a and Mosek 7

n=length(y);
uinx=1:n;
uinx(inx)=[];

pn=length(params);
Cmax=max(cost);
Cmean=mean(cost);

Rmean=zeros(pn,1); % the mean total-cost, i.e., the total-cost w.r.t. Cmean
cisLDMs=cell(pn,1);
for i=1:pn
    C1=params(i).C1;
    C2=params(i).C2;
    lambda1=params(i).lambda1;
    lambda2=params(i).lambda2;
    ker=params(i).ker;
    cisldm=train(x([inx,uinx],:), [y(inx);zeros(length(uinx),1)], Cmax, C1, C2, lambda1, lambda2, ker, op); % minimizing the worst case total-cost
    r=predict(cisldm,x(inx,:),op);  
    cm= confusionMatrix(y(inx),r,[-1,1]);
    Rmean(i)=cm(1,2)+cm(2,1)*Cmean;
    cisLDMs{i}=cisldm;
end
[Rmin, best_lambdas]=min(Rmean); % select the cisldm minimizing the mean total-cost
cisldm=cisLDMs{best_lambdas};