function ldm=train(x, y, Cmax, C1, C2, lambda1, lambda2, ker, op)

% addpath '/Users/stzhou/mosek/7/toolbox/r2013a'
% addpath('libsvm');
addpath 'C:\Program Files\Mosek\7\toolbox\r2013a'
addpath 'libsvm-mat-2.83-1'

% scale the costs
Cpos=2*Cmax/(Cmax+1);
Cneg=2/(Cmax+1);

n = size(x,1);
l_inx = find(y);
u_inx = find(y==0);
ln = length(l_inx);
un = length(u_inx);
%possible number of unlabeled examples whose ground truth are positive
ep=ceil(length(find(y == 1))/ln*un);

if(op==0)
    %Dual Coordinate Descent for regular scale data sets
    %calculate the kernel matrix
    K = kernel(x,x,ker);
    % 1. iteratively estimating the label means
    % 1.1 initialize the label means by supervised SVM using label data only
    opt_svm = ['-t 4 -c ' num2str(C1) ' w1 ' num2str(Cmax)];
    K_l = K(l_inx,l_inx);
    K_l = [(1:ln)',K_l];
    model = svmtrain(y(l_inx),K_l,opt_svm);
    K_t = K(l_inx,u_inx);
    K_t = [(1:un)',K_t'];
    [predict_label, accuracy, dec_values] = svmpredict(ones(un,1),K_t,model);
    if model.Label(1) == -1
        dec_values = -dec_values;
    end
    tmpd = y;
    [val,ix] = sort(dec_values,'descend');
    tmpd(u_inx(ix(1:ep))) = 1;
    tmpd(u_inx(ix(ep+1:un))) = -1;
    % 1.2 fixing label means, solve a SVM problem; fixing SVM, update label means; iteratively until convergence
    iter = 1; flag = 1; maxiter = 100;
    while iter < maxiter && flag
        % 1.2.1 Fixing label means, the SVM problem is a quadratic programming problem whose dual form is as follows,
        var_n = ln+2;   % variable number 
        con_n = 1;      % constraint number
        pos_inx = u_inx(logical(tmpd(u_inx)==1));
        neg_inx = u_inx(logical(tmpd(u_inx)==-1));
        % Hessian matrix, i.e., H = K^{tmpd}.* (\hat{y}*\hat{y}')
        H = zeros(var_n);
        H(1:ln,1:ln) = K(l_inx,l_inx); 
        H(ln+1,1:ln) = mean(K(pos_inx,l_inx));  H(1:ln,ln+1) = H(ln+1,1:ln)';
        H(ln+2,1:ln) = mean(K(neg_inx,l_inx));  H(1:ln,ln+2) = H(ln+2,1:ln)';
        H(ln+1,ln+1) = mean(mean(K(pos_inx,pos_inx)));
        H(ln+1,ln+2) = mean(mean(K(pos_inx,neg_inx))); H(ln+2,ln+1) = H(ln+1,ln+2);
        H(ln+2,ln+2) = mean(mean(K(neg_inx,neg_inx)));      
        tr_y = [y(l_inx);1;-1];
        H = H.* (tr_y*tr_y');
        for i = 1:var_n
            H(i,i) = H(i,i) + 1e-10;
        end
        %linear term f
        f = zeros(var_n,1);
        f(1:ln) = -(Cmax+1)/2*ones(ln,1)-(Cmax-1)/2*y(l_inx);
        %Aeq
        A = zeros(con_n, var_n);
        A(1,ln+1:ln+2) = [Cmax,1];
        %b
        b = C2;
        %lb & ub
        lb = zeros(var_n,1);
        ub = C1*ones(var_n,1);
        ub(ln+1:ln+2) = C2;
        %call quadprog   
        alpha = quadprog(H,f,[],[],A,b,lb,ub);
        % 1.2.2 fix SVM, refine the estimated label means
        % reconstruct the prdiction
        new_alpha = zeros(1,n);
        new_alpha(l_inx) = alpha(1:ln)';
        new_alpha(pos_inx) = alpha(ln+1)/length(pos_inx);
        new_alpha(neg_inx) = alpha(ln+2)/length(neg_inx);      
        pre_val = (new_alpha.*tmpd')*K;
        % estimate the label mean
        tmptmpd = y;
        [val,ix] = sort(pre_val(u_inx),'descend');
        tmptmpd(u_inx(ix(1:ep))) = 1;
        tmptmpd(u_inx(ix(ep+1:un))) = -1;
        % check the stop condition
        if tmptmpd == tmpd
            flag = 0;
        else
            tmpd = tmptmpd; iter = iter + 1;
        end  
    end
    %3. Dual Coordinate Descent
    pos_inx= tmpd==1;
    neg_inx= tmpd==-1;
    c=zeros(n,1);
    c(pos_inx)=Cpos;
    c(neg_inx)=Cneg;
    C=diag(c);
    Q=K+(4*lambda1)*(n*(K*C*C*K)-(K*C*tmpd)*(K*C*tmpd)')/(n^2);  %Q in Eq. 27
    Q=pinv(Q);
    q=-C2*K(:,u_inx)*tmpd(u_inx)-lambda2*K*C*tmpd/n;     %q in Eq. 27
    Y=diag(tmpd(l_inx));
    H1=[Y*K(l_inx,:)*Q*K(:,l_inx)*Y, -Y*K(l_inx,:)*Q*K(:,u_inx), Y*K(l_inx,:)*Q*K(:,u_inx);
        -K(u_inx,:)*Q*K(:,l_inx)*Y, K(u_inx,:)*Q*K(:,u_inx), -K(u_inx,:)*Q*K(:,u_inx);
        K(u_inx,:)*Q*K(:,l_inx)*Y, -K(u_inx,:)*Q*K(:,u_inx), K(u_inx,:)*Q*K(:,u_inx)];
    H1=(H1+H1')/2;
    f1=[-2*Y'*K(l_inx,:)*Q*q-((Cpos+Cneg)/2*ones(ln,1)+(Cpos-Cneg)/2*tmpd(l_inx));
        2*K(u_inx,:)*Q*q+Cpos*ones(un,1);
        -2*K(u_inx,:)*Q*q+Cneg*ones(un,1)];
    ubound=zeros(ln+2*un,1);
    ubound(1:ln) = C1;
    ubound(ln+1:ln+2*un) = C2;
    r=zeros(ln+2*un,1);
    
    h=diag(H1);
    iter = 1; flag = 1; maxiter = 100;
    temp=f1;
    while flag
       r1=r;
       for i=1:ln+2*un
           g=temp(i);
           rold=r(i);
           r(i)=r(i)-g/h(i);
           if(r(i)<0)
               r(i)=0;
           end
           if(r(i)>ubound(i))
               r(i)=ubound(i); 
           end
           temp=temp-(rold-r(i))*H1(:,i);
       end
       o=r-r1;
       s=-o<1e-6 & o<1e-6;
       if(all(s))
           flag=0; 
       else
           iter=iter + 1;
       end
    end
    
    alpha=Q*(K(:,l_inx)*Y*r(1:ln)-K(:,u_inx)*r(ln+1:ln+un)+K(:,u_inx)*r(ln+un+1:ln+2*un)-q);
    ldm.x=x;
    ldm.ker=ker;
    ldm.alpha=alpha;
else
    %Average Stochastic Gradient Descent for large scale data sets
    % bags of little bootstrap
    if(un>10000)
        s=5;
    else
        s=1;
    end
    tempu_inx=u_inx;
    tempx=x;
    tempy=y;
    tempun=un;
    m1=zeros(size(x,2),s);
    m2=zeros(size(x,2),s);
    for turn=1:s
        if(s>1)
            %little bag
            if(1000<=tempun && tempun < 10000)
                usn=ceil(tempun/s);
            else
                usn=ceil(sqrt(tempun)/100)*100;
            end
            u_samp=sampling(1:tempun, usn);
            u_inx=tempu_inx(u_samp);
            x=tempx([l_inx;u_inx],:);
            y=tempy([l_inx;u_inx]);
            u_inx=find(y==0);
            un=length(u_inx);
            ep=ceil(length(find(y == 1))/ln*un);
        end
        % calculate the kernel matrix
        K = kernel(x,x,ker);
        % 1. iteratively estimating the label means
        % 1.1 initialize the label means by supervised SVM using label data only
        opt_svm = ['-t 4 -c ' num2str(C1) ' w1 ' num2str(Cmax)];
        K_l = K(l_inx,l_inx);
        K_l = [(1:ln)',K_l];
        addpath('libsvm-mat-2.83-1')
        model = svmtrain(y(l_inx),K_l,opt_svm);
        K_t = K(l_inx,u_inx);
        K_t = [(1:un)',K_t'];
        [predict_label, accuracy, dec_values] = svmpredict(ones(un,1),K_t,model);
        if model.Label(1) == -1
            dec_values = -dec_values;
        end
        tmpd = y;
        [val,ix] = sort(dec_values,'descend');
        tmpd(u_inx(ix(1:ep))) = 1;
        tmpd(u_inx(ix(ep+1:un))) = -1;
        % 1.2 fixing label means, solve a SVM problem; fixing SVM, update label means; iteratively until convergence
        iter = 1; flag = 1; maxiter = 100;
        while iter < maxiter && flag
            % 1.2.1 Fixing label means, the SVM problem is a quadratic programming problem whose dual form is as follows,
            var_n = ln+2;   % variable number 
            con_n = 1;      % constraint number
            pos_inx = u_inx(logical(tmpd(u_inx)==1));
            neg_inx = u_inx(logical(tmpd(u_inx)==-1));
            % Hessian matrix, i.e., H = K^{tmpd}.* (\hat{y}*\hat{y}')
            H = zeros(var_n);
            H(1:ln,1:ln) = K(l_inx,l_inx); 
            H(ln+1,1:ln) = mean(K(pos_inx,l_inx));  H(1:ln,ln+1) = H(ln+1,1:ln)';
            H(ln+2,1:ln) = mean(K(neg_inx,l_inx));  H(1:ln,ln+2) = H(ln+2,1:ln)';
            H(ln+1,ln+1) = mean(mean(K(pos_inx,pos_inx)));
            H(ln+1,ln+2) = mean(mean(K(pos_inx,neg_inx))); H(ln+2,ln+1) = H(ln+1,ln+2);
            H(ln+2,ln+2) = mean(mean(K(neg_inx,neg_inx)));      
            tr_y = [y(l_inx);1;-1];
            H = H.* (tr_y*tr_y');
            for i = 1:var_n
                H(i,i) = H(i,i) + 1e-10;
            end
            %linear term f
            f = zeros(var_n,1);
            f(1:ln) = -(Cmax+1)/2*ones(ln,1)-(Cmax-1)/2*y(l_inx);
            %Aeq
            A = zeros(con_n, var_n);
            A(1,ln+1:ln+2) = [Cmax,1];
            %b
            b = C2;
            %lb & ub
            lb = zeros(var_n,1);
            ub = C1*ones(var_n,1);
            ub(ln+1:ln+2) = C2;
            %call quadprog   
            alpha = quadprog(H,f,[],[],A,b,lb,ub);
            % 1.2.2 fix SVM, refine the estimated label means
            % reconstruct the prdiction
            pos_inx = u_inx(logical(tmpd(u_inx)==1));
            neg_inx = u_inx(logical(tmpd(u_inx)==-1));
            new_alpha = zeros(1,ln+un);
            new_alpha(l_inx) = alpha(1:ln)';
            new_alpha(pos_inx) = alpha(ln+1)/length(pos_inx);
            new_alpha(neg_inx) = alpha(ln+2)/length(neg_inx);      
            pre_val = (new_alpha.*tmpd')*K;
            % estimate the label mean
            tmptmpd = y;
            [val,ix] = sort(pre_val(u_inx),'descend');
            tmptmpd(u_inx(ix(1:ep))) = 1;
            tmptmpd(u_inx(ix(ep+1:un))) = -1;
            % check the stop condition
            if tmptmpd == tmpd
                flag = 0;
            else
                tmpd = tmptmpd; iter = iter + 1;
            end  
        end
        pos_inx = u_inx(logical(tmpd(u_inx)==1));
        neg_inx = u_inx(logical(tmpd(u_inx)==-1));
        m1(:,turn)=sum(x(pos_inx,:))';
        m2(:,turn)=sum(x(neg_inx,:))';
        clear x;
    end
    %%Average Stochastic Gradient Descent
    x=tempx;
    tmpd=y;
    m=mean(m1,2)-mean(m2,2);
    w=zeros(size(x,2),1);
    w1=zeros(size(x,2),1);
    T=200;
    for t=1:T*ln
        i=sampling(1:n,1);
        j=sampling(1:n,1);
        x1=x(i,:);
        x2=x(j,:);
        g=w1-C2*m+(4*lambda1*x1')*x1*w1-(4*lambda1/(n^2)*m)*m'*w1-lambda2/n*m;
        if(1<=i && i <=ln)
            g=g-8*lambda1/n*tmpd(i)*x1'*m'*w1-lambda2*tmpd(i)*x1';
            if(1<=j && j <=ln)
                g=g-4*lambda1*tmpd(i)*tmpd(j)*x1'*x2*w1;
            end
            if(tmpd(i)*x1*w1<(Cmax+1)/2+tmpd(i)*(Cmax-1)/2)
               g=g-n*C1*tmpd(i)*x1';
            end
        else
            if(x1*w1>Cmax)
                g=g+n*C2*x1';
            end
            if(x1*w1<-1)
                g=g-n*C2*x1';
            end
        end
        eta=0.01/t;
        w1=w1-eta*g;
        if(t-ln<1)
            mu=1; 
        else
            mu=1/(t-ln);
        end
        w=w+mu*(w1-w);
    end
    ldm.w=w;
end
